@@ -41,26 +41,41 @@
 CK_CPPSTART
 #endif
 
-#if defined(__GNUC__) && defined(__GNUC_MINOR__)
-#define GCC_VERSION_AT_LEAST(major, minor) \
+/**
+ * __GNUC_PATCHLEVEL__ is new to GCC 3.0;
+ * it is also present in the widely-used development snapshots leading up to 3.0
+ * (which identify themselves as GCC 2.96 or 2.97, depending on which snapshot you have).
+ *
+ * https://stackoverflow.com/questions/1936719/what-are-the-gcc-predefined-macros-for-the-compilers-version-number/1936745#1936745
+ */
+
+#if defined(__GNUC__) && defined(__GNUC_MINOR__) && defined(__GNUC_PATCHLEVEL__)
+#define GCC_VERSION_AT_LEAST(major, minor, patch) \
+((__GNUC__ > (major)) || \
+ (__GNUC__ == (major) && __GNUC_MINOR__ > (minor)) || \
+ (__GNUC__ == (major) && __GNUC_MINOR__ == (minor) && __GNUC_PATCHLEVEL__ >= (patch)) )
+#elif defined(__GNUC__) && defined(__GNUC_MINOR__)
+#define GCC_VERSION_AT_LEAST(major, minor, patch) \
 ((__GNUC__ > (major)) || \
  (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
 #else
-#define GCC_VERSION_AT_LEAST(major, minor) 0
+#define GCC_VERSION_AT_LEAST(major, minor, patch) 0
 #endif
 
-#if GCC_VERSION_AT_LEAST(2,95)
+#if GCC_VERSION_AT_LEAST(2,95,3)
 #define CK_ATTRIBUTE_UNUSED __attribute__ ((unused))
 #else
 #define CK_ATTRIBUTE_UNUSED
 #endif /* GCC 2.95 */
 
-#if GCC_VERSION_AT_LEAST(2,5)
+#if GCC_VERSION_AT_LEAST(2,5,0)
 #define CK_ATTRIBUTE_NORETURN __attribute__ ((noreturn))
 #else
 #define CK_ATTRIBUTE_NORETURN
 #endif /* GCC 2.5 */
 
+#undef GCC_VERSION_AT_LEAST
+
 #include <sys/types.h>
 
 #if defined(_MSC_VER)
@@ -78,6 +93,15 @@ CK_CPPSTART
  * Used for MSVC to create the export attribute
  * CK_DLL_EXP is defined during the compilation of the library
  * on the command line.
+ *
+ * This definition is only used when building or linking to
+ * the shared library, i.e. libcheck.so. When building the library
+ * the value must be "_declspec(dllexport)".
+ * When linking with the library, the value must be "_declspec(dllimport)"
+ *
+ * This is only used with Microsoft Visual C. In other systems
+ * the value is empty. In MSVC the value is empty when linking with
+ * a static library.
  */
 #ifndef CK_DLL_EXP
 #define CK_DLL_EXP
@@ -86,7 +110,7 @@ CK_CPPSTART
 /* check version numbers */
 
 #define CHECK_MAJOR_VERSION (0)
-#define CHECK_MINOR_VERSION (12)
+#define CHECK_MINOR_VERSION (13)
 #define CHECK_MICRO_VERSION (0)
 
 CK_DLL_EXP extern int CK_EXPORT check_major_version;
@@ -121,6 +145,16 @@ typedef void (*SFun) (void);
  */
 typedef struct Suite Suite;
 
+/**
+ * Type for a test, which wraps a test function
+ */
+typedef struct TTest {
+    const char *name;
+    TFun fn;
+    const char *file;
+    int line;
+} TTest;
+
 /**
  * Creates a test suite with the given name.
  *
@@ -214,8 +248,8 @@ CK_DLL_EXP void CK_EXPORT tcase_set_tags(TCase * tc,
  *
  * @since 0.9.2
  * */
-#define tcase_add_test_raise_signal(tc,tf,signal) \
-   _tcase_add_test((tc),(tf),"" # tf "",(signal), 0, 0, 1)
+#define tcase_add_test_raise_signal(tc,ttest,signal) \
+   _tcase_add_test((tc),(ttest),(signal), 0, 0, 1)
 
 /**
  * Add a test function with an expected exit value to a test case
@@ -229,8 +263,8 @@ CK_DLL_EXP void CK_EXPORT tcase_set_tags(TCase * tc,
  *
  * @since 0.9.7
  */
-#define tcase_add_exit_test(tc, tf, expected_exit_value) \
-  _tcase_add_test((tc),(tf),"" # tf "",0,(expected_exit_value),0,1)
+#define tcase_add_exit_test(tc, ttest, expected_exit_value) \
+  _tcase_add_test((tc),(ttest),0,(expected_exit_value),0,1)
 
 /**
  * Add a looping test function to a test case
@@ -246,8 +280,8 @@ CK_DLL_EXP void CK_EXPORT tcase_set_tags(TCase * tc,
  *
  * @since 0.9.4
  */
-#define tcase_add_loop_test(tc,tf,s,e) \
-  _tcase_add_test((tc),(tf),"" # tf "",0,0,(s),(e))
+#define tcase_add_loop_test(tc,ttest,s,e) \
+  _tcase_add_test((tc),(ttest),0,0,(s),(e))
 
 /**
  * Add a looping test function with signal handling to a test case
@@ -267,8 +301,8 @@ CK_DLL_EXP void CK_EXPORT tcase_set_tags(TCase * tc,
  *
  * @since 0.9.5
  */
-#define tcase_add_loop_test_raise_signal(tc,tf,signal,s,e) \
-  _tcase_add_test((tc),(tf),"" # tf "",(signal),0,(s),(e))
+#define tcase_add_loop_test_raise_signal(tc,ttest,signal,s,e) \
+  _tcase_add_test((tc),(ttest),(signal),0,(s),(e))
 
 /**
  * Add a looping test function with an expected exit value to a test case
@@ -288,16 +322,15 @@ CK_DLL_EXP void CK_EXPORT tcase_set_tags(TCase * tc,
  *
  * @since 0.9.7
  */
-#define tcase_add_loop_exit_test(tc,tf,expected_exit_value,s,e) \
-  _tcase_add_test((tc),(tf),"" # tf "",0,(expected_exit_value),(s),(e))
+#define tcase_add_loop_exit_test(tc,ttest,expected_exit_value,s,e) \
+  _tcase_add_test((tc),(ttest),0,(expected_exit_value),(s),(e))
 
 /* Add a test function to a test case
   (function version -- use this when the macro won't work
 */
-CK_DLL_EXP void CK_EXPORT _tcase_add_test(TCase * tc, TFun tf,
-                                          const char *fname, int _signal,
-                                          int allowed_exit_value, int start,
-                                          int end);
+CK_DLL_EXP void CK_EXPORT _tcase_add_test(TCase * tc, const TTest * ttest,
+                                          int _signal, int allowed_exit_value,
+                                          int start, int end);
 
 /**
  * Add unchecked fixture setup/teardown functions to a test case
@@ -400,16 +433,17 @@ CK_DLL_EXP const char* CK_EXPORT tcase_name(void);
  * @since 0.6.0
  */
 #define START_TEST(__testname)\
-static void __testname (int _i CK_ATTRIBUTE_UNUSED)\
-{\
-  tcase_fn_start (""# __testname, __FILE__, __LINE__);
+static void __testname ## _fn (int _i CK_ATTRIBUTE_UNUSED);\
+static const TTest __testname ## _ttest = {""# __testname, __testname ## _fn, __FILE__, __LINE__};\
+static const TTest * __testname = & __testname ## _ttest;\
+static void __testname ## _fn (int _i CK_ATTRIBUTE_UNUSED)
 
 /**
  *  End a unit test
  *
  * @since 0.6.0
  */
-#define END_TEST }
+#define END_TEST
 
 /*
  * Fail the test case unless expr is false
@@ -770,7 +804,11 @@ do { \
 } while (0)
 
 /**
- * Check two single precision floating point numbers to determine if X == Y
+ * Check two single precision floating point numbers to determine if X == Y.
+ *
+ * Note that the usefulness of this assertion is very limited. If you
+ * want to compare two floating point numbers for equality, you probably
+ * want to use ck_assert_float_eq_tol instead.
  *
  * If not X == Y, the test fails.
  *
@@ -783,7 +821,11 @@ do { \
  */
 #define ck_assert_float_eq(X, Y) _ck_assert_floating(X, ==, Y, float, "")
 /**
- * Check two single precision floating point numbers to determine if X != Y
+ * Check two single precision floating point numbers to determine if X != Y.
+ *
+ * Note that the usefulness of this assertion is very limited. If you
+ * want to compare two floating point numbers for equality, you probably
+ * want to use ck_assert_float_ne_tol instead.
  *
  * If not X != Y, the test fails.
  *
@@ -969,7 +1011,11 @@ do { \
 #define ck_assert_float_nonnan(X) _ck_assert_floating_nonnan(X, float, "")
 
 /**
- * Check two double precision floating point numbers to determine if X == Y
+ * Check two double precision floating point numbers to determine if X == Y.
+ *
+ * Note that the usefulness of this assertion is very limited. If you
+ * want to compare two floating point numbers for equality, you probably
+ * want to use ck_assert_double_eq_tol instead.
  *
  * If not X == Y, the test fails.
  *
@@ -982,7 +1028,11 @@ do { \
  */
 #define ck_assert_double_eq(X, Y) _ck_assert_floating(X, ==, Y, double, "")
 /**
- * Check two double precision floating point numbers to determine if X != Y
+ * Check two double precision floating point numbers to determine if X != Y.
+ *
+ * Note that the usefulness of this assertion is very limited. If you
+ * want to compare two floating point numbers for equality, you probably
+ * want to use ck_assert_double_ne_tol instead.
  *
  * If not X != Y, the test fails.
  *
@@ -1168,7 +1218,11 @@ do { \
 #define ck_assert_double_nonnan(X) _ck_assert_floating_nonnan(X, double, "")
 
 /** 
- * Check two double precision floating point numbers to determine if X == Y
+ * Check two double precision floating point numbers to determine if X == Y.
+ *
+ * Note that the usefulness of this assertion is very limited. If you
+ * want to compare two floating point numbers for equality, you probably
+ * want to use ck_assert_ldouble_eq_tol instead.
  *
  * If not X == Y, the test fails.
  *
@@ -1181,7 +1235,11 @@ do { \
  */
 #define ck_assert_ldouble_eq(X, Y) _ck_assert_floating(X, ==, Y, long double, "L")
 /**
- * Check two double precision floating point numbers to determine if X != Y
+ * Check two double precision floating point numbers to determine if X != Y.
+ *
+ * Note that the usefulness of this assertion is very limited. If you
+ * want to compare two floating point numbers for equality, you probably
+ * want to use ck_assert_ldouble_ne_tol instead.
  *
  * If not X != Y, the test fails.
  *
@@ -1405,8 +1463,8 @@ do { \
 /**
  * Check two strings to determine if 0==strcmp(X,Y)
  *
- * If X or Y is NULL the test failes.
- * If (0==strcmp(X,Y)), the test fails.
+ * If X or Y is NULL the test fails.
+ * If not 0==strcmp(X,Y), the test fails.
  *
  * @param X string
  * @param Y string to compare against X
